/* ====================== RegisterUser32() ========================
 * Uses Reginald's FUNCDEF() function to register some useful
 * functions in the Windows operating system file USER32.DLL. These
 * mostly have to do with managing actual GUI windows.
 *
 * err = RegisterUser32(functions)
 *
 * functions = A listing of the names of the USER32 functions
 *             you wish registered, each separated by a space.
 *             For example, the string "GetDC FillRect"
 *             would register those two functions. If omitted,
 *             then all USER32 functions are registered.
 *
 * RETURNS: An empty string if success, or an error message.
 *
 * NOTE: This uses the SOURCELINE() function, so be careful about
 * editing anything in this file (before the first FUNCDEF).
 */
RegisterUser32: PROCEDURE
SIGNAL ON FAILURE NAME RegisterUser32Error
SIGNAL ON SYNTAX NAME RegisterUser32Error
OPTIONS "WINFUNC"
functions = TRANSLATE(ARG(3))
RECT = "32,32,32,32"
PAINTSTRUCT = "void *,32,struct RECT,32,32,8u[32]"
POINT = "32,32"
DRAWTEXTPARAMS = "32u,32,32,32,32u"
WNDENUMPROC="32,void,void"
PROPENUMPROC="32,void,str,void,void"
WNDCLASSEX="32u,32u,void,32,32,void,void,void,void,str *,str *,void"
WINDOWPLACEMENT="32u,32u,32u,struct POINT,struct POINT,struct RECT"
SCROLLINFO="32u,32u,32,32,32u,32,32"
MENUITEMINFO="32u,32u,32u,32u,32u,void,void,void,32u,str *,32u"
MSG="void,32u,32u,32u,32u,32,32"
ICONINFO="32u,32u,32u,void,void"
TIMERPROC=",void,32u,32u,32u"
WINDOWPROC="void,void,32u,void,void"
IF functions = "" THEN DO
CALL FUNCDEF "GetDC","32u, void","user32"
CALL FUNCDEF "ReleaseDC","32,void,void","user32"
CALL FUNCDEF "BeginPaint","32u,void,void stor","user32"
CALL FUNCDEF "EndPaint","32,void,void","user32"
CALL FUNCDEF "DrawText","32u,void,str,32u,struct RECT,32u","user32"
CALL FUNCDEF "DrawTextEx","32u,void,str,32u,struct RECT,32u,struct DRAWTEXTPARAMS","user32"
CALL FUNCDEF "InvalidateRect","32,void,struct RECT,32","user32"
CALL FUNCDEF "GetClientRect","32,void,struct RECT stor","user32"
CALL FUNCDEF "RedrawWindow","32,void,struct RECT,void,32u","user32"
CALL FUNCDEF "UpdateWindow","32,void","user32"
CALL FUNCDEF "CopyFromRect","32,struct RECT stor,void","user32","CopyRect"
CALL FUNCDEF "CopyToRect","32,void,struct RECT","user32","CopyRect"
CALL FUNCDEF "SetRect","32,void,32,32,32,32","user32"
CALL FUNCDEF "FillRect","32,void,struct RECT,void","user32"
CALL FUNCDEF "OffsetRect","32,void,32,32","user32"
CALL FUNCDEF "InflateRect", "32,void,32,32","user32"
CALL FUNCDEF "SubtractRect","32,struct RECT,void,void","user32"
CALL FUNCDEF "GetWindowDC","32u,void","user32"
CALL FUNCDEF "FindWindowEx","void,void,void,str,str","user32"
CALL FUNCDEF "WindowFromDC","void,void","user32"
CALL FUNCDEF "PtInRect","32,void,struct POINT","user32"
CALL FUNCDEF "EnableWindow","32,void,32","user32"
CALL FUNCDEF "MoveWindow","32,void,32,32,32,32,32","user32"
CALL FUNCDEF "ShowWindow","32,void,32","user32"
CALL FUNCDEF "DestroyIcon","32,void","user32"
CALL FUNCDEF "IsWindowEnabled","32,void","user32"
CALL FUNCDEF "IsWindowVisible","32,void","user32"
CALL FUNCDEF "ClientToScreen","32,void,struct POINT dual","user32"
CALL FUNCDEF "ScreenToClient","32,void,void","user32"
CALL FUNCDEF "SendMessage","void,void,32u,void,void","user32"
CALL FUNCDEF "GetTabbedTextExtent","32u,void,str,32u,32u,void","user32"
CALL FUNCDEF "TabbedTextOut","32u,void,32,32,str,32u,void,32","user32"
CALL FUNCDEF "SetWindowPos","32,void,void,32,32,32,32,32u","user32"
CALL FUNCDEF "SetWindowPlacement","32,void,struct WINDOWPLACEMENT","user32"
CALL FUNCDEF "GetForegroundWindow","void","user32"
CALL FUNCDEF "GetActiveWindow","void","user32"
CALL FUNCDEF "GetCapture","void","user32"
CALL FUNCDEF "SetCapture","void,void","user32"
CALL FUNCDEF "ReleaseCapture","32","user32"
CALL FUNCDEF "SetForegroundWindow","32,void","user32"
CALL FUNCDEF "IsWindowEnabled","32,void","user32"
CALL FUNCDEF "GetMessage","32,struct MSG stor,void,32u,32u","user32"
CALL FUNCDEF "TranslateMessage","32,struct MSG","user32"
CALL FUNCDEF "WaitMessage","32","user32"
CALL FUNCDEF "GetSysColorBrush","32u,32","user32"
CALL FUNCDEF "SetParent","void,void,void","user32"
CALL FUNCDEF "GetClipboardData","void,32u","user32"
CALL FUNCDEF "GetClipboardFormatNameA","32u,32u,str[256] stor,32u","user32" /* Hardwired to 256 byte buffer */
CALL FUNCDEF "GetClipboardOwner","void","user32"
CALL FUNCDEF "GetClipboardViewer","void","user32"
CALL FUNCDEF "GetOpenClipboardWindow","void","user32"
CALL FUNCDEF "IsClipboardFormatAvailable","32,32u","user32"
CALL FUNCDEF "OpenClipboard","32,void","user32"
CALL FUNCDEF "RegisterClipboardFormat","32u,str","user32"
CALL FUNCDEF "SetClipboardData","void,32u,void","user32"
CALL FUNCDEF "SetClipboardViewer","void,void","user32"
CALL FUNCDEF "LoadCursorFromFile","void,str","user32"
CALL FUNCDEF "LoadStdCursor","void,void,32u","user32","LoadCursor"
CALL FUNCDEF "LoadImage","32,void,str,32u,32,32,32u","user32"
CALL FUNCDEF "DrawIconEx","32,void,32,32,void,32,32,32u,void,32u","user32"
CALL FUNCDEF "GetNextWindow","void,void,32","user32"
CALL FUNCDEF "GetParent","void,void","user32"
CALL FUNCDEF "CheckMenuRadioItem","32,void,32u,32u,32u,32u","user32"
CALL FUNCDEF "CreateMenu","void","user32"
CALL FUNCDEF "CreatePopupMenu","void","user32"
CALL FUNCDEF "DeleteMenu","32,void,32u,32u","user32"
CALL FUNCDEF "DestroyMenu","32,void","user32"
CALL FUNCDEF "DrawMenuBar","32,void","user32"
CALL FUNCDEF "EnableMenuItem","32,void,32u,32u","user32"
CALL FUNCDEF "GetMenu","void,void","user32"
CALL FUNCDEF "GetMenuDefaultItem","32u,void,32u,32u","user32"
CALL FUNCDEF "GetMenuItemCount","32,void","user32"
CALL FUNCDEF "GetMenuItemID","32u,void,32","user32"
CALL FUNCDEF "GetMenuItemInfo","32,void,32u,32,struct MENUITEMINFO stor","user32"
CALL FUNCDEF "SetMenuItemInfo","32,void,32u,32,struct MENUITEMINFO","user32"
CALL FUNCDEF "GetSubMenu","void,void,32","user32"
CALL FUNCDEF "GetMenuItemRect","32,void,void,32u,struct RECT stor","user32"
CALL FUNCDEF "GetSystemMenu","void,void,32","user32"
CALL FUNCDEF "InsertMenuItem","32,void,32u,32,struct MENUITEMINFO","user32"
CALL FUNCDEF "IsMenu","32,void","user32"
CALL FUNCDEF "MenuItemFromPoint","32u,void,void,struct POINT","user32"
CALL FUNCDEF "RemoveMenu","32,void,32u,32u","user32"
CALL FUNCDEF "SetMenu","32,void,void","user32"
CALL FUNCDEF "SetMenuDefaultItem","32,void,32u,32u","user32"
CALL FUNCDEF "SetMenuItemBitmaps","32,void,32u,32u,void,void","user32"
CALL FUNCDEF "TrackPopupMenu","32,void,32u,32,32,32,void,struct RECT","user32"
CALL FUNCDEF "ClipCursor","32,struct RECT","user32"
CALL FUNCDEF "CopyCursor","void,void","user32"
CALL FUNCDEF "CopyIcon","void,void","user32"
CALL FUNCDEF "CopyImage","void,void,32u,32,32,32u","user32"
CALL FUNCDEF "DrawFrameControl","32,void,struct RECT,32u,32u","user32"
CALL FUNCDEF "DrawCaption",",void,struct RECT,void,void,str,16u","user32"
CALL FUNCDEF "DrawEdge","32,void,struct RECT,32u,32u","user32"
CALL FUNCDEF "DrawFocusRect","32,void,struct RECT","user32"
CALL FUNCDEF "DrawState","32,void,void,void,void,void,32,32,32,32,32u","user32"
CALL FUNCDEF "DrawStateText","32,void,void,void,str,void,32,32,32,32,32u","user32"
CALL FUNCDEF "EqualRect","32,void,struct RECT","user32"
CALL FUNCDEF "ExcludeUpdateRgn","32,void,void","user32"
CALL FUNCDEF "FrameRect","32,void,void","user32"
CALL FUNCDEF "InvalidateRgn","32,void,void,32u","user32"
CALL FUNCDEF "InvertRect","32,void,struct RECT","user32"
CALL FUNCDEF "IsRectEmpty","32,void","user32"
CALL FUNCDEF "GetUpdateRect","32,void,struct RECT stor,32","user32"
CALL FUNCDEF "GetUpdateRgn","32,void,void,32","user32"
CALL FUNCDEF "GetWindowRgn","32,void,void","user32"
CALL FUNCDEF "GrayString","32,void,void,void,str,32u,32,32,32u,32u","user32"
CALL FUNCDEF "IntersectRect","32,struct RECT stor,void,void","user32"
CALL FUNCDEF "SetRectEmpty","32, void","user32"
CALL FUNCDEF "SetWindowRgn","32,void,void,32","user32"
CALL FUNCDEF "UnionRect","32,struct RECT,void,void","user32"
CALL FUNCDEF "ValidateRect","32,void,struct RECT","user32"
CALL FUNCDEF "ValidateRgn","32,void,void","user32"
CALL FUNCDEF "MapWindowPoints","32u,void,void,void,32u","user32"
CALL FUNCDEF "FlashWindow","32,void,32","user32"
CALL FUNCDEF "MessageBeep","32,32u","user32"
CALL FUNCDEF "WaitForInputIdle","32,void,32u","user32"
CALL FUNCDEF "GetSysColor","32u,32","user32"
CALL FUNCDEF "ArrangeIconicWindow","32u,void","user32"
CALL FUNCDEF "BeginDeferWindowPos","void,32","user32"
CALL FUNCDEF "BringWindowToTop","32,void","user32"
CALL FUNCDEF "CallWindowProc","void,void,void,32u,32u,32u","user32"
CALL FUNCDEF "ChildWindowFromPointEx","void,void,struct POINT,32u","user32"
CALL FUNCDEF "CreateWindowEx","void,32u,str,str,32u,32,32,32u,32u,void,32u,void,void","user32"
CALL FUNCDEF "DeferWindowPos","void,void,void,void,32,32,32u,32u,32u","user32"
CALL FUNCDEF "DefWindowProc","void,void,32u,void,void","user32"
CALL FUNCDEF "DestroyWindow","32,void","user32"
CALL FUNCDEF "EndDeferWindowPos","32,void","user32"
CALL FUNCDEF "EnumChildWindows","32,void,func WNDENUMPROC,void","user32"
CALL FUNCDEF "EnumPropsEx","32,void,func PROPENUMPROC,void","user32"
CALL FUNCDEF "EnumThreadWindows","32,32u,func WNDENUMPROC,void","user32"
CALL FUNCDEF "EnumWindows","32,func WNDENUMPROC,void","user32"
CALL FUNCDEF "GetClassInfoEx","32,void,str,struct WNDCLASSEX stor","user32"
CALL FUNCDEF "GetClassLong","32u,void,32","user32"
CALL FUNCDEF "GetClassName","32u,void,str[256] stor,32u","user32"
CALL FUNCDEF "GetDesktopWindow","void","user32"
CALL FUNCDEF "GetLastActivePopup","void,void","user32"
CALL FUNCDEF "GetProp","void,void,str","user32"
CALL FUNCDEF "GetTopWindow","void,void","user32"
CALL FUNCDEF "GetWindow","void,void,32","user32"
CALL FUNCDEF "GetWindowProc","func WINDOWPROC,void,32","user32","GetWindowLong"
CALL FUNCDEF "GetWindowLong","void,void,32","user32"
CALL FUNCDEF "SetWindowProc","func WINDOWPROC,void,32,func WINDOWPROC","user32","SetWindowLong"
CALL FUNCDEF "SetWindowLong","void,void,32,void","user32"
CALL FUNCDEF "GetWindowPlacement","32,void,struct WINDOWPLACEMENT","user32"
CALL FUNCDEF "GetWindowRect","32,void,struct RECT stor","user32"
CALL FUNCDEF "GetWindowText","32u,void,32u","user32"
CALL FUNCDEF "GetWindowTextLength","32u,void","user32"
CALL FUNCDEF "GetWindowThreadProcessId","32u,void,32 * stor","user32"
CALL FUNCDEF "IsChild","32,void,void","user32"
CALL FUNCDEF "IsIconic","32,void","user32"
CALL FUNCDEF "IsWindow","32,void","user32"
CALL FUNCDEF "IsWindowUnicode","32,void","user32"
CALL FUNCDEF "IsZoomed","32,void","user32"
CALL FUNCDEF "OpenIcon","32,void","user32"
CALL FUNCDEF "RegisterClassEx","void,struct WNDCLASSEX","user32"
CALL FUNCDEF "RemoveProp","void,void,str","user32"
CALL FUNCDEF "SetClassLong","32u,32,void","user32"
CALL FUNCDEF "SetProp","32,void,str,void","user32"
CALL FUNCDEF "SetWindowText","32,void,str","user32"
CALL FUNCDEF "ShowOwnedPopups","32,void,32","user32"
CALL FUNCDEF "ShowWindowAsync","32,void,32","user32"
CALL FUNCDEF "UnregisterClass","32,str,void","user32"
CALL FUNCDEF "WindowFromPoint","void,struct POINT","user32"
CALL FUNCDEF "GetSystemMetrics","32,32","user32"
CALL FUNCDEF "SetCaretPos","32,32,32","user32"
CALL FUNCDEF "SetCaretBlinkTime","32,32u","user32"
CALL FUNCDEF "SetCursor","void,void","user32"
CALL FUNCDEF "SetCursorPos","32,32,32","user32"
CALL FUNCDEF "SetSystemCursor","32,void,32u","user32"
CALL FUNCDEF "ShowCaret","32,void","user32"
CALL FUNCDEF "ShowCursor","32,32","user32"
CALL FUNCDEF "MsgWaitForMultipleObjects","32u,32u,void,32,32u,32u","user32"
CALL FUNCDEF "ExitWindowsEx","32,32u,32u","user32"
CALL FUNCDEF "WinHelp","32,void,str,32u,void","user32"
CALL FUNCDEF "CloseClipboard","32","user32"
CALL FUNCDEF "CountClipboardFormats","32u","user32"
CALL FUNCDEF "EmptyClipboard","32","user32"
CALL FUNCDEF "EnumClipboardFormats","32u,32u","user32"
CALL FUNCDEF "CheckDlgButton","32,void,32,32u","user32"
CALL FUNCDEF "CheckRadioButton","32,void,32,32,32","user32"
CALL FUNCDEF "EnableScrollBar","32,void,32u,32u","user32"
CALL FUNCDEF "GetScrollInfo","32,void,32,struct SCROLLINFO stor","user32"
CALL FUNCDEF "GetScrollPos","32,void,32","user32"
CALL FUNCDEF "GetScrollRange","32,void,32,32u *, 32u *","user32"
CALL FUNCDEF "IsDlgButtonChecked","32u,void,32u","user32"
CALL FUNCDEF "ScrollDC","32,void,32,32,struct RECT,struct RECT,void,struct RECT stor","user32"
CALL FUNCDEF "ScrollWindowEx","32,void,32,32,struct RECT,struct RECT,void,struct RECT stor,32u","user32"
CALL FUNCDEF "SetScrollInfo","32,void,32,struct SCROLLINFO,32","user32"
CALL FUNCDEF "SetScrollPos","32,void,32,32,32","user32"
CALL FUNCDEF "SetScrollRange","32,void,32,32,32,32","user32"
CALL FUNCDEF "ShowScrollBar","32,void,32,32","user32"
CALL FUNCDEF "GetAsyncKeyState","16,32","user32"
CALL FUNCDEF "GetFocus","void","user32"
CALL FUNCDEF "SetActiveWindow","void,void","user32"
CALL FUNCDEF "SetFocus","void,void","user32"
CALL FUNCDEF "GetInputState","32","user32"
CALL FUNCDEF "DispatchMessage","32,struct MSG","user32"
CALL FUNCDEF "PeekMessage","32,struct MSG stor,void,32u,32u,32u","user32"
CALL FUNCDEF "PostMessage","32,void,32u,void,void","user32"
CALL FUNCDEF "GetMessagePos","32u","user32"
CALL FUNCDEF "GetMessageTime","32u","user32"
CALL FUNCDEF "GetQueueStatus","32u,32u","user32"
CALL FUNCDEF "InSendMessage","32","user32"
CALL FUNCDEF "PostQuitMessage",",32","user32"
CALL FUNCDEF "PostThreadMessage","32,32u,32u,void,void","user32"
CALL FUNCDEF "RegisterWindowMessage","32u,str","user32"
CALL FUNCDEF "ReplyMessage","32,void","user32"
CALL FUNCDEF "SendMessageTimeout","void,void,32u,void,void,32u,32u,32u * stor","user32"
CALL FUNCDEF "SendNotifyMessage","void,void,32u,void,void","user32"
CALL FUNCDEF "GetCaretPos","32,struct POINT *","user32"
CALL FUNCDEF "GetClipCursor","32,struct RECT stor","user32"
CALL FUNCDEF "GetCursor","void","user32"
CALL FUNCDEF "GetCursorPos","32,struct POINT *","user32"
CALL FUNCDEF "GetIconInfo","32,void,struct ICONINFO stor","user32"
CALL FUNCDEF "HideCaret","32,void","user32"
CALL FUNCDEF "CreateCaret","32,void,void,32,32","user32"
CALL FUNCDEF "DestroyCaret","32","user32"
CALL FUNCDEF "DestroyCursor","32,void","user32"
CALL FUNCDEF "KillTimer","32,void,32u","user32"
CALL FUNCDEF "SetTimer","32u,void,32u,32u,func TIMERPROC","user32"
CALL FUNCDEF "AdjustWindowRectEx","32,struct RECT dual,32u,32u,32u","user32"
CALL FUNCDEF "LoadString","32u,void,32u,void,32u","user32"
END
ELSE DO
   DO WHILE functions \= ""
      PARSE VAR functions func functions
      IF func \= "" THEN DO i = 39 TO 260
         PARSE UPPER VALUE SOURCELINE(i) WITH 'CALL FUNCDEF "' try '"'
         IF func = try THEN DO
            INTERPRET SOURCELINE(i)
            LEAVE i
         END
      END
   END
END
RETURN ""
RegisterUser32Error:
RAISE ERROR (CONDITION('E')) DESCRIPTION (CONDITION('D')) RETURN (CONDITION('D'))